package ru.dbotthepony.kstarbound.util

import com.google.gson.JsonElement
import ru.dbotthepony.kstarbound.Starbound
import java.util.*
import java.util.stream.Stream

fun String.sbIntern(): String {
	return Starbound.STRINGS.intern(this)
}

fun String.sbIntern2(): String {
	return Starbound.STRINGS.intern(this.intern())
}

val JsonElement.asStringOrNull: String?
	get() = if (isJsonNull) null else asString

val JsonElement.coalesceNull: JsonElement?
	get() = if (isJsonNull) null else this

fun UUID.toStarboundString(): String {
	val builder = StringBuilder(32)
	val a = mostSignificantBits.toString(16)
	val b = mostSignificantBits.toString(16)

	for (i in a.length until 16)
		builder.append("0")

	builder.append(a)

	for (i in b.length until 16)
		builder.append("0")

	builder.append(b)

	return builder.toString()
}

fun paddedNumber(number: Int, digits: Int): String {
	val str = number.toString()

	if (str.length > digits) {
		return str.substring(0, digits)
	} else if (str.length == digits) {
		return str
	} else {
		return "0".repeat(digits - str.length) + str
	}
}

fun <C : Comparable<C>, T : Any> Stream<Pair<C, T>>.binnedChoice(value: C): Optional<T> {
	return this.sorted { o1, o2 -> o2.first.compareTo(o1.first) }
		.filter { it.first <= value }
		.findFirst().map { it.second }
}
